use std::io::{self, Read};

use tokio::sync::mpsc;
use uuid::Uuid;

use crate::{
    config::FelinetConfig,
    felinet::PacketIn as SemiPacketIn,
    util::{append_internet_to_packet_route, print_packet},
};

use self::{demod::Demod, parse::LeF32Decoder};

mod demod;
mod packet;
mod parse;

trait Bit {
    fn bit(&self) -> bool;
}

impl Bit for f32 {
    fn bit(&self) -> bool {
        *self < 0.0
    }
}

pub fn decode_forever(
    felinet_send: mpsc::Sender<SemiPacketIn>,
    felinet_config: &Option<FelinetConfig>,
    uuid: &Uuid,
) {
    let stdin = io::stdin();
    let soft_bits = LeF32Decoder::new(stdin.bytes().map(|x| x.expect("Could not read from stdin")));
    let demod = Demod::new(soft_bits);

    demod.demod(|mut p| {
        print_packet(&p);

        if let Some(fc) = felinet_config {
            if append_internet_to_packet_route(&fc.callsign, fc.ssid, &mut p).is_none() {
                return;
            };

            if let Ok(semi) = p.semi_encode() {
                let semi = SemiPacketIn {
                    raw: semi.to_vec(),
                    uuid: (*uuid).into(),
                };

                let _ = felinet_send.blocking_send(semi);
            }
        }
    });
}
